/*
	$Id: ExtraValueExpression.java 6127 2012-09-14 07:28:35Z nicklas $

	Copyright (C) 2005 Nicklas Nordborg
	Copyright (C) 2006 Jari Häkkinen, Nicklas Nordborg, Martin Svensson
	
	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.core;

import java.util.Collection;

import net.sf.basedb.core.query.Expression;
import net.sf.basedb.core.query.Query;
import net.sf.basedb.core.query.QueryElement;
import net.sf.basedb.core.query.QueryType;

/**
	Represents the value of an extra value in a dynamic query:
	<code>alias.value</code>.

	@author Nicklas
	@version 2.0
	@base.modified $Date: 2005-11-03 16:09:12 +0100 (to, 03 nov 2005) $
*/
public class ExtraValueExpression
	implements Expression
{
	
	private final ExtraValue extraValue;
	private final VirtualColumn column;
	
	/**
		Create a new expression for the specified extra value. It is assumed
		that the extra value has been joined before it is used in an expression.
		@param extraValue The extra value 
		@see net.sf.basedb.core.ExtraValueJoin
		@see net.sf.basedb.core.query.Dynamic#extraValue(ExtraValue)
	*/
	public ExtraValueExpression(ExtraValue extraValue)
	{
		this.extraValue = extraValue;
		this.column = extraValue.getVirtualColumn();
	}
	
	/*
		From the QueryElement interface
		-------------------------------------------
	*/
	@Override
	public String toQl(Query query, DbControl dc)
		throws BaseException
	{
		if (query.getQueryType() == QueryType.SQL)
		{
			String alias = HibernateUtil.quote(extraValue.getJoinAlias());
			return alias + "." + HibernateUtil.quote(column.getColumn());
		}
		else
		{
			throw new UnsupportedOperationException("ExtraValueExpression operation not supported by "+query.getQueryType()+" query.");
		}
	}
	/**
		@return Always null
	*/
	@Override
	public Collection<? extends QueryElement> getChildren()
	{
		return null;
	}
	// -------------------------------------------
	/*
		From the Object class
		-------------------------------------------
	*/
	@Override
	public String toString()
	{
		return extraValue.getJoinAlias()+"." + column.getName();
	}
	/**
		This expression is equal to another ExtraValueExpression if they 
		represents the same extra value.
	*/
	@Override
	public boolean equals(Object other)
	{
		if (this == other) return true;
		if (other == null || this.getClass() != other.getClass()) return false;
		ExtraValueExpression o = (ExtraValueExpression)other;
		return extraValue.equals(o.extraValue);
	}
	@Override
	public int hashCode()
	{
		return extraValue.hashCode() + getClass().hashCode();
	}
	// -------------------------------------------

	public ExtraValue getExtraValue()
	{
		return extraValue;
	}
}
