/*
  $Id: AnnotationSetData.java 6917 2015-05-25 12:10:03Z nicklas $

	Copyright (C) 2005 Nicklas Nordborg
	Copyright (C) 2006 Jari Häkkinen, Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

  BASE is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 3
  of the License, or (at your option) any later version.

  BASE is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.core.data;

import java.io.Serializable;
import java.util.Set;
import java.util.HashSet;

/**
	This class holds information about a set of annotations.
	
	@author Nicklas
	@version 2.0
	@see net.sf.basedb.core.AnnotationSet
	@see <a href="../../../../../../html/developer/api/data_api.annotations.html">Developer documentation: Annotations</a>
	@hibernate.class table="`AnnotationSets`" lazy="true" batch-size="10"
 */
public class AnnotationSetData
	extends BasicData
	implements LoggableData, Serializable
{

	private static final long serialVersionUID = 7582258239220452911L;

	public AnnotationSetData()
	{}

	private transient int itemType;
	/**
		Get the item type this annotation set is associated with.
		@hibernate.property column="`item_type`" type="int" not-null="true" update="false"
	*/
	public int getItemType()
	{
		return itemType;
	}
	public void setItemType(int itemType)
	{
		this.itemType = itemType;
	}

	private transient Integer itemId;
	/**
		The ID if the item this annotation set is associated with. Can be null
		if this is not know. Use a query based on the itemType and annotation set 
		ID to find the item in this case.
		@since 2.14
		@hibernate.property column="`item_id`" type="int" not-null="false"
	*/
	@NotLoggable
	public Integer getItemId()
	{
		return itemId;
	}
	public void setItemId(Integer itemId)
	{
		this.itemId = itemId;
	}
	
	private transient Set<AnnotationData> annotations;
	/**
		The annotations in this annotation set. This is the inverse end.
		@see AnnotationData#getAnnotationSet()
		
		@hibernate.set lazy="true" cascade="delete" inverse="true"
		@hibernate.collection-key column="`annotationset_id`"
		@hibernate.collection-one-to-many class="net.sf.basedb.core.data.AnnotationData"
	*/
	@NotLoggable
	public Set<AnnotationData> getAnnotations()
	{
		if (annotations == null)
		{
			annotations = new HashSet<AnnotationData>();
		}
		return annotations;
	}
	void setAnnotations(Set<AnnotationData> annotations)
	{
		this.annotations = annotations;
	}
	
}
