/*
	$Id: GuiContextData.java 6870 2015-04-16 11:47:41Z nicklas $

	Copyright (C) 2005 Nicklas Nordborg
	Copyright (C) 2006 Jari Häkkinen, Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.core.data;

import java.io.Serializable;

/**
	This is a helper component for the {@link PluginDefinitionData} item to
	store the contexts where a plugin can be used in a client application.
	A context is an item type and context type code. The context type code can currently
	take two values: 0 = list; 1 = item

	@author Nicklas
	@version 2.0
	@see <a href="../../../../../../html/developer/api/data_api.plugins.html">Developer documentation: Plug-ins, jobs and job agents</a>
	@base.modified $Date: 2015-04-16 13:47:41 +0200 (to, 16 apr 2015) $
*/
public class GuiContextData
	implements Serializable
{
	/**
		
	*/
	private static final long serialVersionUID = 8098514351740554692L;
	GuiContextData()
	{}
	
	public GuiContextData(int itemType, int contextType)
	{
		this.itemType = itemType;
		this.contextType = contextType;
	}

	private int itemType;
	/**
	 	@hibernate.property column="`item_type`" type="int"
	*/
	public int getItemType()
	{
		return itemType;
	}
	void setItemType(int itemType)
	{
		this.itemType = itemType;
	}

	private int contextType;
	/**
	 	@hibernate.property column="`context_type`" type="int"
	*/
	public int getContextType()
	{
		return contextType;
	}
	void setContextType(int contextType)
	{
		this.contextType = contextType;
	}

	/**
		Check if this object is equal to another <code>GuiContext</code>
		object. They are equal if both have the same item type and context type
	*/
	@Override
	public boolean equals(Object o)
	{
		if ((o == null) || (getClass() != o.getClass()))
		{
			return false;
		}
		GuiContextData gc = (GuiContextData)o;
		return (this.itemType == gc.itemType) && (this.contextType == gc.contextType);
	}

	/**
		Calculate the hash code for the object.
	*/
	@Override
	public int hashCode()
	{
		return 53 * itemType + 17 * contextType;
	}
}
