package net.sf.basedb.core.query;

import java.util.Collection;

import net.sf.basedb.core.BaseException;
import net.sf.basedb.core.DbControl;
import net.sf.basedb.core.Item;
import net.sf.basedb.core.Permission;

/**
	Restriction that inserts an "impossible" filter requirement (1=0)
	if the logged in user is lacking a given permission. Note that
	if the given permission is DENIED then 1=0 is returned if the
	user has that permission, otherwise if the user is lacking the permission.
	
	@author nicklas
	@since 3.6.2
*/
public class PermissionRestriction 
	implements Restriction 
{

	private final Permission permission;
	private final Item itemType;

	public PermissionRestriction(Permission permission, Item itemType) 
	{
		this.itemType = itemType;
		this.permission = permission;
	}
	
	@Override
	public String toQl(Query query, DbControl dc) 
		throws BaseException 
	{
		boolean hasPermission = dc.getSessionControl().hasPermission(permission, itemType);
		boolean checkDenied = permission == Permission.DENIED;
		return hasPermission == checkDenied ? "1=0" : "1=1";
	}

	@Override
	public Collection<? extends QueryElement> getChildren() 
	{
		return null;
	}

}
