/**
	$Id: FilterUtil.java 4594 2008-10-21 07:20:40Z nicklas $

	Copyright (C) 2008 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.util.filter;

import java.util.ArrayList;
import java.util.Collection;
import java.util.LinkedList;
import java.util.List;

/**
	Utility class for working with and using filters.

	@author Nicklas
	@version 2.9
	@base.modified $Date: 2008-10-21 09:20:40 +0200 (ti, 21 okt 2008) $
*/
public class FilterUtil
{
	/**
		Filter a collection and return a new collection with the
	 	objects that passes the {@link Filter#evaluate(Object)} method.
		If the original collection is null, null is returned. If the
		original collection is empty, or if no objects passes the filter
		an empty list is returned. If the filter is null a list with all objects
		of the original collection is returned. The original collection
		is never modifed.
		
		@param objects The collection of objects
		@param filter The filter
		@return A new collection, or null if the original collection is null
		@since 2.9
	*/
	public static <T> List<T> filter(Collection<? extends T> objects, Filter<? super T> filter)
	{
		List<T> list = null;
		if (objects != null)
		{
			if (filter == null)
			{
				list = new ArrayList<T>(objects);
			}
			else
			{
				list = new LinkedList<T>();
				for (T object : objects)
				{
					if (filter.evaluate(object)) list.add(object);
				}
			}
		}
		return list;
	}
}
