/**
	$Id $

	Copyright (C) 2015 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.util.overview.loader;

import net.sf.basedb.core.BasicItem;
import net.sf.basedb.core.BioPlateEvent;
import net.sf.basedb.core.DbControl;
import net.sf.basedb.core.DerivedBioAssay;
import net.sf.basedb.core.MeasuredBioMaterial;
import net.sf.basedb.core.PhysicalBioAssay;
import net.sf.basedb.core.Item;
import net.sf.basedb.core.Kit;
import net.sf.basedb.core.PermissionDeniedException;
import net.sf.basedb.util.overview.Node;
import net.sf.basedb.util.overview.OverviewContext;
import net.sf.basedb.util.overview.node.ChildNodeDirection;
import net.sf.basedb.util.overview.node.NameableNameGenerator;
import net.sf.basedb.util.overview.node.NodeFactory;

/**
	Node loader implementation for kits. Kits are loaded as
	end-point property nodes with no children. It can't be a root 
	node or a forward/reverse-loading node.
	
	@author Nicklas
	@since 3.7
*/
public class KitLoader
	extends BasicItemNodeLoader<Kit>
{
	
	public KitLoader()
	{
		super(Item.KIT, DENY_ROOT_NODE, 
				new NameableNameGenerator<Kit>("kit", "Kit"));
	}
	
	/*
		From the NodeLoader interface
		------------------------------
	*/
	@Override
	public Node createPropertyNode(DbControl dc, OverviewContext context, Node parentNode)
	{
		Kit kit = null;
		boolean denied = false;
		try
		{
			BasicItem parentItem = parentNode.getItem(dc);
			if (parentItem instanceof DerivedBioAssay)
			{
				kit = ((DerivedBioAssay)parentItem).getKit();
			}
			else if (parentItem instanceof PhysicalBioAssay)
			{
				kit = ((PhysicalBioAssay)parentItem).getCreationEvent().getKit();
			}
			else if (parentItem instanceof MeasuredBioMaterial)
			{
				kit = ((MeasuredBioMaterial)parentItem).getCreationEvent().getKit();
			}
			else if (parentItem instanceof BioPlateEvent)
			{
				kit = ((BioPlateEvent)parentItem).getKit();
			}
			// If there are other items with a kit
			// we should add more cases here
		}
		catch (PermissionDeniedException ex)
		{
			denied = true;
		}
		
		NodeFactory<Kit> nf = getNodeFactory(dc, context);
		Node hardwareNode = createItemNode(nf, kit, kit, denied, 
				parentNode, ChildNodeDirection.PROPERTY);
		return hardwareNode;
	}
	// -----------------------------------
	/*
		From the AbstractNodeLoader class
		----------------------------------
	*/
	/**
		Loads annotations for the given kit node.
		@see RawBioAssayLoader#createForwardNode(DbControl, OverviewContext, Node)
	*/
	@Override
	protected void loadPropertyChildNodes(DbControl dc, OverviewContext context, Node kitNode)
	{
		getNodeLoader(context, Item.ANNOTATION).createPropertyNode(dc, context, kitNode);
	}
	// -------------------------------------	
}
