/*
	$Id: TestGenericOverview.java 6100 2012-09-04 08:43:21Z nicklas $

	Copyright (C) 2009 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.
	
	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.
	
	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
import java.util.List;

import net.sf.basedb.core.BasicItem;
import net.sf.basedb.core.DataFileType;
import net.sf.basedb.core.DbControl;
import net.sf.basedb.core.DerivedBioAssay;
import net.sf.basedb.core.Extract;
import net.sf.basedb.core.Hardware;
import net.sf.basedb.core.Item;
import net.sf.basedb.core.PhysicalBioAssay;
import net.sf.basedb.core.Platform;
import net.sf.basedb.core.Protocol;
import net.sf.basedb.core.Quantity;
import net.sf.basedb.core.SessionControl;
import net.sf.basedb.core.Software;
import net.sf.basedb.core.SystemItems;
import net.sf.basedb.core.Type;
import net.sf.basedb.util.overview.Failure;
import net.sf.basedb.util.overview.GenericOverview;
import net.sf.basedb.util.overview.Node;
import net.sf.basedb.util.overview.OverviewUtil;
import net.sf.basedb.util.overview.loader.BasicItemNodeLoaderFactory;

public class TestGenericOverview
{
	static boolean ok = true;

	public static void main(String[] args)
	{
		TestUtil.checkArgs(args);
		TestUtil.begin();
		ok = test_all();
		TestUtil.stop();
	}

	static boolean test_all()
	{
		write("++Testing generic overview");
		// Create support items
		// Annotation types
		int annotationType = TestAnnotationType.test_create("Source", Type.STRING, 0, 
				null, null, Item.RAWBIOASSAY, 1, new String[] {"alfa", "beta"}, true);

		// Protocols
		int scanningProtocol = TestProtocol.test_create(
				SystemItems.getId(Protocol.SCANNING), "Scan protocol", false);
		int sequencingProtocol = TestProtocol.test_create(
				SystemItems.getId(Protocol.SEQUENCING), "Sequencing protocol", false);
		int featureProtocol = TestProtocol.test_create(
				SystemItems.getId(Protocol.FEATURE_EXTRACTION), "Feature protocol", false);
		int power = SystemItems.getId(Quantity.POWER);
		int milliWatt = TestUnit.test_load_by_symbol(power, "mW");
		int scanPower = TestAnnotationType.test_create("Power", Type.FLOAT, milliWatt, 
				null, null, Item.DERIVEDBIOASSAY, 1, null, false);
		TestProtocol.test_set_protocol_parameter(scanningProtocol, scanPower);

		// Hardware and software
		int scanner = TestHardware.test_create(
				SystemItems.getId(Hardware.SCANNER), "Scanner GO", false);
		int sequencer = TestHardware.test_create(
				SystemItems.getId(Hardware.SEQUENCER), "Sequencer GO", false);
		int software = TestSoftware.test_create(
				SystemItems.getId(Software.FEATURE_EXTRACTION), "Feature extraction GO", false);
		int alignmentSoftware = TestSoftware.test_create(
				SystemItems.getId(Software.ALIGNMENT), "Alignment GO", false);
		
		// Data files
		int genePixFile = TestFile.test_create("data/test.rawdata.import.txt", false, false);
		int celFile = TestFile.test_create("data/test.affymetrix.cel", false, false);
		int reporterMap = TestFile.test_create("data/test", false, false);
		int samFile = TestFile.test_create("data/test.assembly.sam", false, false);

		// Create main items
		
		// Biomaterial
		int biosource = TestBioSource.test_create("Test GO", false);
		int sample1 = TestSample.test_create(biosource, "Sample GO #1", 0, false);
		int sample2 = TestSample.test_create(biosource, "Sample GO #2", 0, false);
		int extract1 = TestExtract.test_create(sample1, "Extract GO #1", 0, 0, false);
		int extract2 = TestExtract.test_create(sample2, "Extract GO #2", 0, 0, false);
		int extractPooled = TestExtract.test_create_pooled("Extract GO #3 (pooled)", extract1, extract2);
		
		// Array design
		int arrayDesign = TestArrayDesign.test_create(Platform.GENERIC, false);
		TestArrayDesign.test_set_file(arrayDesign, DataFileType.GENERIC_REPORTER_MAP, reporterMap);
		
		// Array batch & slide
		int arrayBatch = TestArrayBatch.test_create(arrayDesign, false);
		int arraySlide1 = TestArraySlide.test_create(arrayBatch, false);
		int arraySlide2 = TestArraySlide.test_create(arrayBatch, false);
		
		// ******** Microarray part *******
		// Labeled extracts
		int cy3 = TestTag.test_load("cy3");
		int cy5 = TestTag.test_load("cy5");
		int labeledExtract = SystemItems.getId(Extract.LABELED);
		int labeledExtract1 = TestExtract.test_create(extract1, "Labeled GO #1 (cy3)", labeledExtract, cy3, false);
		int labeledExtract2 = TestExtract.test_create(extract2, "Labeled GO #2 (cy3)", labeledExtract, cy3, false);
		int labeledExtract1Ref = TestExtract.test_create(extractPooled, "Labeled GO #1 (cy5)", labeledExtract, cy5, false);
		int labeledExtract2Ref = TestExtract.test_create(extractPooled, "Labeled GO #2 (cy5)", labeledExtract, cy5, false);
		
		// Hybridization
		int hyb1 = TestPhysicalBioAssay.test_create(arraySlide1, "Hyb GO #1", SystemItems.getId(PhysicalBioAssay.HYBRIDIZATION), 1);
		TestPhysicalBioAssay.test_add_extract(hyb1, labeledExtract1, 1, null);
		TestPhysicalBioAssay.test_add_extract(hyb1, labeledExtract1Ref, 1, null);

		int hyb2 = TestPhysicalBioAssay.test_create(arraySlide2, "Hyb GO #2", SystemItems.getId(PhysicalBioAssay.HYBRIDIZATION), 1);
		TestPhysicalBioAssay.test_add_extract(hyb2, labeledExtract2, 1, null);
		TestPhysicalBioAssay.test_add_extract(hyb2, labeledExtract2Ref, 1, null);

		// Scan
		int scan1 = TestDerivedBioAssay.test_create_root("Scan GO #1", hyb1, 
				SystemItems.getId(DerivedBioAssay.SCAN), 0, scanningProtocol, scanner, 0);
		TestAnnotation.test_annotatate(Item.DERIVEDBIOASSAY, scan1, scanPower, 0, 15.0f);
		int scan2 = TestDerivedBioAssay.test_create_root("Scan GO #2", hyb2, 
				SystemItems.getId(DerivedBioAssay.SCAN), 0, scanningProtocol, scanner, 0);
		TestAnnotation.test_annotatate(Item.DERIVEDBIOASSAY, scan2, scanPower, 0, 15.0f);

		// Raw bioassay #1
		int rawBioAssay1 = TestRawBioAssay.test_create(Platform.GENERIC, "genepix", "RBA #1", scan1, 0, software, featureProtocol, arrayDesign, false);
		TestDataFileType.test_set_file(Item.RAWBIOASSAY, rawBioAssay1, DataFileType.GENERIC_RAW_DATA, genePixFile);
		TestDataFileType.test_set_file(Item.RAWBIOASSAY, rawBioAssay1, DataFileType.AFFYMETRIX_CEL, celFile);
		TestAnnotation.test_annotatate(Item.RAWBIOASSAY, rawBioAssay1, annotationType, 0, "alfa");

		// Raw bioassay #2
		int rawBioAssay2 = TestRawBioAssay.test_create(Platform.GENERIC, "genepix", "RBA #2", scan2, 0, software, featureProtocol, arrayDesign, false);
		TestDataFileType.test_set_file(Item.RAWBIOASSAY, rawBioAssay2, DataFileType.GENERIC_RAW_DATA, genePixFile);

		// **** Sequencing part ********
		// Libraries
		int library = SystemItems.getId(Extract.LIBRARY);
		int library1 = TestExtract.test_create(extract1, "Library GO #1", library, 0, false);
		int library2 = TestExtract.test_create(extract2, "Library GO #2", library, 0, false);
		
		// Flow cell
		int flowCell = TestPhysicalBioAssay.test_create(0, "Flow cell GO #1", SystemItems.getId(PhysicalBioAssay.CLONAL_AMPLIFICATION), 2);
		TestPhysicalBioAssay.test_add_extract(flowCell, library1, 1, null);
		TestPhysicalBioAssay.test_add_extract(flowCell, library2, 2, null);
		
		// Sequencing
		int sequencing = TestDerivedBioAssay.test_create_root("Sequenced GO", flowCell, 
			SystemItems.getId(DerivedBioAssay.SEQUENCES), 0, sequencingProtocol, sequencer, 0);
		
		int alignedAssayId1 = TestDerivedBioAssay.test_create_child("Aligned GO #1", library1,
			SystemItems.getId(DerivedBioAssay.ARRANGEMENT), 0, 0, alignmentSoftware, sequencing);
		int alignedAssayId2 = TestDerivedBioAssay.test_create_child("Aligned GO #2", library2,
				SystemItems.getId(DerivedBioAssay.ARRANGEMENT), 0, 0, alignmentSoftware, sequencing);
		TestDataFileType.test_set_file(Item.DERIVEDBIOASSAY, alignedAssayId1, DataFileType.SAM, samFile);
		TestDataFileType.test_set_file(Item.DERIVEDBIOASSAY, alignedAssayId2, DataFileType.SAM, samFile);

		// Raw bioassays 
		int seqRaw1 = TestRawBioAssay.test_create(Platform.GENERIC, "genepix", "SeqRBA #1", alignedAssayId1, library1, software, 0, arrayDesign, false);
		int seqRaw2 = TestRawBioAssay.test_create(Platform.GENERIC, "genepix", "SeqRBA #2", alignedAssayId2, library2, software, 0, arrayDesign, false);
		
		// **** Experiment ******
		int experiment = TestExperiment.test_create("genepix", false);
		TestExperiment.test_add_rawbioassay(experiment, rawBioAssay1, rawBioAssay2, seqRaw1, seqRaw2);
		TestExperiment.test_add_experimental_factors(experiment, scanPower, annotationType);

		// Show some overviews
		test_overview(Item.BIOSOURCE, biosource);
		test_overview(Item.PHYSICALBIOASSAY, hyb1);
		test_overview(Item.PHYSICALBIOASSAY, flowCell);
		test_overview(Item.ARRAYDESIGN, arrayDesign);
		test_overview(Item.EXPERIMENT, experiment);
		
		// Delete items
		if (TestUtil.waitBeforeDelete()) TestUtil.waitForEnter();
		// Main items
		TestExperiment.test_delete(experiment);
		
		TestRawBioAssay.test_delete(seqRaw1);
		TestRawBioAssay.test_delete(seqRaw2);
		TestDerivedBioAssay.test_delete(alignedAssayId1);
		TestDerivedBioAssay.test_delete(alignedAssayId2);
		TestDerivedBioAssay.test_delete(sequencing);
		TestPhysicalBioAssay.test_delete(flowCell);
		TestExtract.test_delete(library1);
		TestExtract.test_delete(library2);
		
		TestRawBioAssay.test_delete(rawBioAssay1);
		TestRawBioAssay.test_delete(rawBioAssay2);
		TestDerivedBioAssay.test_delete(scan1);
		TestDerivedBioAssay.test_delete(scan2);
		TestPhysicalBioAssay.test_delete(hyb1);
		TestPhysicalBioAssay.test_delete(hyb2);
		TestArraySlide.test_delete(arraySlide1);
		TestArraySlide.test_delete(arraySlide2);
		TestArrayBatch.test_delete(arrayBatch);
		TestArrayDesign.test_delete(arrayDesign);
		TestExtract.test_delete(labeledExtract1);
		TestExtract.test_delete(labeledExtract1Ref);
		TestExtract.test_delete(labeledExtract2);
		TestExtract.test_delete(labeledExtract2Ref);
		
		TestExtract.test_delete(extractPooled);
		TestExtract.test_delete(extract1);
		TestExtract.test_delete(extract2);
		TestSample.test_delete(sample1);
		TestSample.test_delete(sample2);
		TestBioSource.test_delete(biosource);
		
		// Support items
		TestSoftware.test_delete(software);
		TestSoftware.test_delete(alignmentSoftware);
		TestHardware.test_delete(scanner);
		TestHardware.test_delete(sequencer);
		TestProtocol.test_delete(featureProtocol);
		TestProtocol.test_delete(scanningProtocol);
		TestProtocol.test_delete(sequencingProtocol);
		TestFile.test_delete(celFile);
		TestFile.test_delete(genePixFile);
		TestFile.test_delete(reporterMap);
		TestFile.test_delete(samFile);
		TestAnnotationType.test_delete(scanPower);
		TestAnnotationType.test_delete(annotationType);
		
		write("++Testing generic overview "+(ok ? "OK" : "Failed")+"\n");
		return ok;
	}
	

	
	static void write(String message)
	{
		System.out.println(message);
	}
	static void write(GenericOverview go)
	{
		if (TestUtil.getSilent()) return;
		write("--Generating overview...");
		write(go.getRootNode(), "");
		int i = 0;
		for (Failure f : go.getFailures())
		{
			++i;
			write("--failure " + i + ": " + f.getMessage() + " on node " + f.getNode().getTitle());
		}
	}
	static void write(Node node, String indent)
	{
		write("  " + indent + node.getName() + ": " + node.getTitle());
		List<Node> children = node.getChildren();
		if (children != null)
		{
			for (Node child : children)
			{
				write(child, "".equals(indent) ? "|-" : "| " + indent);
			}
		}
	}
	
	
	static void test_overview(Item itemType, int id)
	{
		if (id == 0) return;
		DbControl dc = null;
		SessionControl sc = TestUtil.getSessionControl();
		try
		{
			dc = TestUtil.getDbControl();
			BasicItem item = itemType.getById(dc, id);
			
			GenericOverview go = OverviewUtil.getNewOverview(dc, item, null);
			((BasicItemNodeLoaderFactory)go.getNodeLoaderFactory()).setUseNullLoaderIfNotFound(true);
			go.expand(dc, go.getRootNode(), true);
			write(go);
			
			write("--Load generic overview [root=" + item + "] OK");
		}
		catch (Throwable ex)
		{
			write("--Load generic overview [root =" + itemType + "(" + id + ") FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
	}

	

}
