<%-- $Id: list_directories.jsp 6605 2014-11-18 12:48:42Z nicklas $
	------------------------------------------------------------------
	Copyright (C) 2006 Jari Häkkinen, Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
	------------------------------------------------------------------

	@author Nicklas
	@version 2.0
--%>
<%@ page pageEncoding="UTF-8" session="false"
	import="net.sf.basedb.core.SessionControl"
	import="net.sf.basedb.core.DbControl"
	import="net.sf.basedb.core.SystemItems"
	import="net.sf.basedb.core.Item"
	import="net.sf.basedb.core.ItemContext"
	import="net.sf.basedb.core.Directory"
	import="net.sf.basedb.core.Experiment"
	import="net.sf.basedb.core.User"
	import="net.sf.basedb.core.ItemQuery"
	import="net.sf.basedb.core.ItemResultIterator"
	import="net.sf.basedb.core.Permission"
	import="net.sf.basedb.core.query.Restrictions"
	import="net.sf.basedb.core.query.Expressions"
	import="net.sf.basedb.core.query.Orders"
	import="net.sf.basedb.core.query.Hql"
	import="net.sf.basedb.util.FileUtil"
	import="net.sf.basedb.clients.web.Base"
	import="net.sf.basedb.clients.web.util.HTML"
	import="net.sf.basedb.util.Values"
	import="java.util.Map"
	import="java.util.Set"
	import="java.util.HashSet"
	import="java.util.List"
	import="org.json.simple.JSONArray"
	import="org.json.simple.JSONObject"
%>
<%@ taglib prefix="base" uri="/WEB-INF/base.tld" %>
<%@ taglib prefix="tbl" uri="/WEB-INF/table.tld" %>
<%!

void generateSubTree(JSONObject jsonParent, Directory parent, Map<Directory, List<Directory>> tree, Set<Directory> ignore)
{
	if (ignore.contains(parent)) return;
	ignore.add(parent);

	List<Directory> children = tree.get(parent);
	if (children == null) return;
	
	for (Directory child : children)
	{
		if (ignore.contains(child)) continue;
		String folderIcon = "Folder";
		if (child.isHomeDirectory())
		{
			folderIcon = "UserHome";
		}
		else if (child.isRemoved())
		{
			folderIcon = "FolderDeleted";
		}
		else if (child.getAutoCompress())
		{
			folderIcon = "FolderCompressed";
		}
		
		JSONObject jsonDir = newJoustEntry(jsonParent, folderIcon, HTML.encodeTags(child.getName()), Integer.toString(child.getId()));
		jsonDir.put("path", child.getPath().toString());
		jsonDir.put("isLazy", 1);
		generateSubTree(jsonDir, child, tree, ignore);
	}
}
JSONObject newJoustEntry(JSONObject jsonParent, String icon, String text, String id)
{
	JSONObject jsonJoust = new JSONObject();
	jsonJoust.put("icon", icon);
	jsonJoust.put("text", text);
	jsonJoust.put("id", id);
	if (jsonParent != null)
	{
		JSONArray jsonChildren = (JSONArray)jsonParent.get("children");
		if (jsonChildren == null)
		{
			jsonChildren = new JSONArray();
			jsonParent.put("children", jsonChildren);
			jsonParent.remove("isLazy");
		}
		jsonChildren.add(jsonJoust);
	}
	return jsonJoust;
}
%>
<%
final Item itemType = Item.DIRECTORY;
final SessionControl sc = Base.getExistingSessionControl(pageContext, Permission.DENIED, itemType);
final String ID = sc.getId();
final DbControl dc = sc.newDbControl();
final String mode = request.getParameter("mode");
final String callback = request.getParameter("callback");
final float scale = Base.getScale(sc);
final String requestTitle = request.getParameter("title");
final boolean syncWithDirectory = Values.getBoolean(request.getParameter("sync"), true);
final ItemContext cc = sc.getCurrentContext(itemType);
int directoryId = cc.getId();
try
{
	final User user = User.getById(dc, sc.getLoggedInUserId());
	final Directory userHome = user.getHomeDirectory();
	final boolean readDirectories = sc.hasPermission(Permission.READ, Item.DIRECTORY);
	final Directory root = Directory.getById(dc, SystemItems.getId(readDirectories ? Directory.ROOT : Directory.HOME));
	
	Directory current = null;
	// Try to get current/last visited directory. If not successful, use default directory.
	try
	{
		current = directoryId != 0 ? Directory.getById(dc, directoryId) : null;
	}
	catch (Throwable ex){}	
	if (current == null) current = userHome;
	if (current == null) current = root;
	
	final String rootIcon;
	final String rootTitle;
	if (root.isRootDirectory())
	{
		rootIcon = "Root";
		rootTitle = "Root";
	}
	else
	{
		rootIcon = "Home";
		rootTitle = "Other users";
	}
	Map<Directory, List<Directory>> tree = FileUtil.loadMinimalDirectoryTree(dc, root, userHome, current);
	Set<Directory> ignore = new HashSet<Directory>();
	
	// Create directory tree
	JSONArray json = new JSONArray();

	// Search results -- hidden to begin with
	JSONObject jsonSearch = newJoustEntry(null, "Search", "Search results", "search");
	jsonSearch.put("noOutlineIcon" , 1);
	jsonSearch.put("className", "search");
	json.add(jsonSearch);
	
	// User home directory
	if (userHome != null)
	{
		JSONObject jsonHome = newJoustEntry(null, "Home", "My home", Integer.toString(userHome.getId()));
		jsonHome.put("noOutlineIcon" , 1);
		jsonHome.put("isOpen", "1");
		jsonHome.put("path", userHome.getPath().toString());
		json.add(jsonHome);
		generateSubTree(jsonHome, userHome, tree, ignore);
	}
	
	// Main root directory
	JSONObject jsonRoot = newJoustEntry(null, rootIcon, HTML.encodeTags(rootTitle), Integer.toString(root.getId()));
	jsonRoot.put("noOutlineIcon" , 1);
	jsonRoot.put("isOpen" , 1);
	jsonRoot.put("path", root.getPath().toString());
	json.add(jsonRoot);
	generateSubTree(jsonRoot, root, tree, ignore);

	// Add 'current' as root entry if we have not seen it before
	if (!ignore.contains(current))
	{
		// but... move up as far as the user has 'read' permission...
		Directory currentRoot = current;
		try
		{
			while (currentRoot != null && !ignore.contains(currentRoot.getParent()))
			{
				currentRoot = currentRoot.getParent();
			}
		}
		catch (Throwable t)
		{}
		if (currentRoot == null) currentRoot = current;
		JSONObject jsonCurrentRoot = newJoustEntry(null, "Folder", HTML.encodeTags(currentRoot.getPath().toString()), Integer.toString(currentRoot.getId()));
		jsonCurrentRoot.put("noOutlineIcon" , 1);
		jsonCurrentRoot.put("isOpen" , 1);
		jsonCurrentRoot.put("path", currentRoot.getPath().toString());
		json.add(jsonCurrentRoot);
		generateSubTree(jsonCurrentRoot, currentRoot, tree, ignore);
	}
	%>
	<base:page title="<%=requestTitle != null ? requestTitle : ""%>" 
		type="<%="selectonedirectory".equals(mode) ? "popup" : "iframe"%>" id="list-page">
	<base:head scripts="joust-2.js,dragdrop.js,~directories.js" styles="joust-2.css,toolbar.css">
	<style>
	#search
	{
		display:none; 
		font-style: italic; 
		border-bottom: 1px solid #cccccc; 
		margin-bottom: 2px; 
		padding-bottom: 2px;
	}
	</style>
	</base:head>
	
	<base:body>
		<div id="page-data" class="datacontainer"
			data-mode="<%=mode == null ? "" : mode%>"
			data-sync-with-directory="<%=syncWithDirectory ? 1 : 0%>"
			data-initial-directory="<%=current.getId() %>"
		></div>
		<%
		if ("selectonedirectory".equals(mode))
		{
			%>
				<h1><%=requestTitle == null ? "Select one directory" : requestTitle%></h1>
				
				<form name="directory">
				<input type="hidden" name="directory_id" value="<%=current.getId()%>">
				<input type="hidden" name="callback" value="<%=callback%>">
	
				<div class="content bottomborder">
					<div id="main" class="absolutefull" style="bottom: 3em;">
						<tbl:toolbar subclass="bottomborder bg-filled-50">
							<tbl:button 
								id="btnNewDirectory"
								image="directory_new.png" 
								title="New&hellip;" 
								tooltip="Create a new directory" 
							/>
						</tbl:toolbar>
						<div id="joust" class="joust absolutefull" 
							data-joust-tree="<%=HTML.encodeTags(json.toJSONString()) %>"
							style="overflow: auto; top: 1.8em;"></div>
					</div>
				
					<div class="absolutefull topborder" style="top: auto; height: 3em;">
						<table class="fullform input100">
							<tr>
								<th><%=requestTitle == null ? "Path" : requestTitle%></th>
								<td><input type="text" class="text" readonly name="path"></td>
							</tr>
						</table>
					</div>
				</div>
				</form>
	
				<base:buttongroup subclass="dialogbuttons">
					<base:button id="btnOk" title="Ok" />
					<base:button id="close" title="Cancel" />
				</base:buttongroup>
			<%
		}
		else
		{
			%>
			<div id="main" class="absolutefull auto-init" data-auto-init="drag-support" style="overflow: hidden;">
				<tbl:toolbar subclass="bottomborder bg-filled-50">
					<tbl:button 
						id="btnRefresh"
						image="refresh.png" 
						title="Refresh" 
						tooltip="Refresh the directory tree" 
					/>
					<tbl:button 
						id="btnSearch"
						image="search.png" 
						title="Search" 
						tooltip="Search for files and directories" 
					/>
				</tbl:toolbar>
				<div id="joust" class="joust absolutefull" 
					data-joust-tree="<%=HTML.encodeTags(json.toJSONString()) %>"
					style="overflow: auto; top: 2em;"></div>
			</div>
			<%
		}
		%>
		</base:body>
	</base:page>
	<%
}
finally
{
	if (dc != null) dc.close();
}
%>
