/* $Id $
	------------------------------------------------------------------
	Copyright (C) 2013 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
	------------------------------------------------------------------

	@author Nicklas
*/
'use strict';

var SimpleExport = function()
{
	var simpleExport = {};
	var tableId;
	var selectedItems;
	
	/**
		Initialize the page.
	*/
	simpleExport.initPage = function()
	{
		// Works as long as we don't have more than one list per page
		tableId = window.opener.document.forms[0].name; 
		selectedItems = window.opener.Table.getSelected(tableId);
		var pageable = window.opener.document.forms[0].rowsperpage ? true : false;

		Events.addEventHandler('formatText', 'change', simpleExport.formatChanged);
		Events.addEventHandler('formatXml', 'change', simpleExport.formatChanged);
		Events.addEventHandler('collectionSeparatorPreset', 'change', simpleExport.separatorPresetChanged);
		
		Events.addEventHandler('presets', 'change', simpleExport.updateColumns);
		
		Buttons.addClickHandler('moveUp', simpleExport.moveExportedUpOrDown);
		Buttons.addClickHandler('moveDown', simpleExport.moveExportedUpOrDown);
		
		Buttons.addClickHandler('moveLeft', simpleExport.moveBetweenLists);
		Buttons.addClickHandler('moveRight', simpleExport.moveBetweenLists);
		Events.addEventHandler('exported', 'dblclick', simpleExport.moveBetweenLists);
		Events.addEventHandler('not_exported', 'dblclick', simpleExport.moveBetweenLists);
		
		Buttons.addClickHandler('btnBrowse', simpleExport.browseOnClick);
		Events.addEventHandler('path', 'base-selected', simpleExport.setSaveAsCallback);
		
		Buttons.addClickHandler('btnShowError', simpleExport.showErrorList);
		Buttons.addClickHandler('btnHideError', simpleExport.hideErrorList);
		
		Buttons.addClickHandler('close', simpleExport.cancelWizard);
		Buttons.addClickHandler('btnExport', simpleExport.doExport);
		
		// Initialize
		simpleExport.updateColumns();
		var frm = document.forms['export'];
		if (selectedItems.length == 0)
		{
			frm['parameter:whichItems'][0].disabled = true;
			var lblSelected = Doc.element('lblSelected');
			lblSelected.disabled = true;
			Doc.addClass(lblSelected, 'disabled');
		}
		if (!pageable)
		{
			frm['parameter:whichItems'][1].disabled = true;
			var lblCurrentPage = Doc.element('lblCurrentPage');
			lblCurrentPage.disabled = true;
			Doc.addClass(lblCurrentPage, 'disabled');
		}

	}
	
	/**
		Cancel export wizard.
	*/
	simpleExport.cancelWizard = function()
	{
		location.href = 'index.jsp?ID='+App.getSessionId()+'&cmd=CancelWizard';
	}
	
	/**
		Enable/disable collection separator input.
	*/
	simpleExport.formatChanged = function()
	{
		var textFormat = Doc.element('formatText').checked;
		Doc.element('collectionSeparator').disabled = !textFormat;
		Doc.element('collectionSeparatorPreset').disabled = !textFormat;
	}
	
	/**
		Set the collection separator from a preset.
	*/
	simpleExport.separatorPresetChanged = function(event)
	{
		Doc.element('collectionSeparator').value = event.currentTarget.value;
		event.currentTarget.selectedIndex = 0;
	}
	
	/**
		Move the selected options in the 'exported columns' list up or down.
	*/
	simpleExport.moveExportedUpOrDown = function(event)
	{
		var moveDown = Data.int(event.currentTarget, 'down');
		var moved = Forms.moveListOptions('exported', moveDown);
		if (moved > 0) simpleExport.clearPreset();
	}

	/**
		Move the selected options between exported and not exported lists.
	*/
	simpleExport.moveBetweenLists = function(event)
	{
		var fromList = Data.get(event.currentTarget, 'from') || event.currentTarget;
		var toList = Data.get(event.currentTarget, 'to');
		var moved = Forms.moveSelected(fromList, toList);
		if (moved > 0) simpleExport.clearPreset();
	}
	
	// Update the settings according to the selection in the presets list
	simpleExport.updateColumns = function()
	{
		var frm = document.forms['export'];
		var columns = frm.presets.value;
		if (columns == '') return;
		if (columns == '_default_') columns = Data.get('page-data', 'default-columns');
		if (columns == '_current_') columns = window.opener.Table.getColumns(tableId);
		simpleExport.initColumns(columns);
	}
	
	simpleExport.clearPreset = function()
	{
		var frm = document.forms['export'];
		frm.presets.selectedIndex = 0;
	}
	
	simpleExport.initColumns = function(columns)
	{
		var frm = document.forms['export'];
		var exported = frm.exported;
		var notExported = frm.not_exported;
		exported.length = 0;
		notExported.length = 0;

		var columnDefs = window.opener.Table.getColumnDefs(tableId);
		// Move all columns into the not exported or exported list
		var list = columns == 'all' ? exported : notExported;
		for (var i = 0; i < columnDefs.length; i++)
		{
			var col = columnDefs[i];
			if (col.exportable)
			{
				var option = new Option(col.title, col.id);
				list.options[list.options.length] = option;
			}
		}
		if (columns != 'all')
		{
			// Scan columns and move back those that should be exported
			var cols = columns.split(',');
			for (var ic = 0; ic < cols.length; ic++) // >
			{
				var col = columnDefs['id'+cols[ic]];
				if (col)
				{
					for (var ih = 0; ih < notExported.length; ih++) // >
					{
						if (col.id == notExported[ih].value)
						{
							var option = new Option(col.title, col.id);
							exported[exported.length] = option;
							notExported[ih] = null;
							ih = notExported.length;
						}
					}
				}
			}
		}
	}

	simpleExport.browseOnClick = function()
	{
		var url = App.getRoot() +'filemanager/index.jsp?ID='+App.getSessionId();
		url += '&cmd=SaveAs&callback=path';

		var frm = document.forms['export'];
		var i = frm.path.value.lastIndexOf('/');
		if (i >= 0) 
		{
			url += '&fileName=' + encodeURIComponent(frm.path.value.substring(i+1));
		}

		Dialogs.openPopup(url, 'SaveAs', 1050, 700);
	}
	
	simpleExport.setSaveAsCallback = function(event)
	{
		var frm = document.forms['export'];
		frm.path.value = event.detail.path;
	}

	
	simpleExport.hideErrorList = function()
	{
		Doc.hide('errorlist');
		Doc.show('showerrorlist');
	}
	
	simpleExport.showErrorList = function()
	{
		Doc.show('errorlist');
		Doc.hide('showerrorlist');
	}

	simpleExport.doExport = function()
	{
		var frm = document.forms['export'];
		var exported = document.forms['export'].exported;
		if (exported.length == 0)
		{
			Forms.showNotification(exported, 'You must select at least one column to export');
			return;
		}
		
		// Send each selected column as: property:type:title		
		var columnDefs = window.opener.Table.getColumnDefs(tableId);
		for (i = 0; i < exported.length; i++)
		{
			var colDef = columnDefs['id'+exported[i].value];
			
			var col = colDef.exportProperty;
			if (col.indexOf(':') == -1)
			{
				col += ':' + colDef.dataType;
			}
			col += ':' + colDef.title;
			Forms.addHidden(frm, 'parameter:whichProperties', col);
		}
		
		// If only selected items should be exported, create parameter for each one
		if (Forms.getCheckedRadio(frm['parameter:whichItems']).value == 'selected')
		{
			for (var i = 0; i < selectedItems.length; i++)
			{
				Forms.addHidden(frm, 'parameter:item', selectedItems[i]);
			}
		}
		Forms.addHidden(frm, 'parameter:sameUnits', frm.sameUnits.checked ? "true" : "false");
		if (frm.path.value != '')
		{
			Forms.addHidden(frm, 'parameter:saveAs', frm.path.value);
		}
		frm.submit();
	}
	
	return simpleExport;
}();

Doc.onLoad(SimpleExport.initPage);

