/* $Id $
	------------------------------------------------------------------
	Copyright (C) 2013 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
	------------------------------------------------------------------

	@author Nicklas
*/
'use strict';

var Reporters = function()
{
	var reporters = {};
	
	/**
		Initialize the page.
	*/
	reporters.initPage = function()
	{
		var pageId = Doc.getPageId();
		if (pageId == 'list-page')
		{
			var attributes = {'item-type': 'REPORTER'};
			var tableAttributes = {'table-id': 'reporters'};
			Buttons.addClickHandler('btnAddReporters', reporters.addReporters);
			Buttons.addClickHandler('btnRemoveReporters', reporters.removeReporters);
			Buttons.addClickHandler('btnColumns', Buttons.configureColumns, tableAttributes);
			Buttons.addClickHandler('btnExport', Buttons.runListPlugin, tableAttributes);
			Buttons.addClickHandler('btnImport', Buttons.runListPlugin, tableAttributes);
			Buttons.addClickHandler('btnRunPlugin', Buttons.runListPlugin, tableAttributes);

			Buttons.addClickHandler('close', App.closeWindow);
			Buttons.addClickHandler('btnOk', Buttons.returnSelected, tableAttributes);

			TabControl.addTabActivateListener('main.properties', reporters.viewReporterList);
		}
		else if (pageId == 'add-page')
		{
			Buttons.addClickHandler('btnAddSelected', reporters.addSelected);
			Buttons.addClickHandler('btnAddCurrentPage', reporters.addCurrentPage);
			Buttons.addClickHandler('btnAddAll', reporters.addAll);
			Buttons.addClickHandler('close', App.closeWindow);
		}
	}
	
	reporters.addReporters = function()
	{
		var frm = document.forms['reporters'];
		var url = 'add_reporters_frameset.jsp?ID='+App.getSessionId();
		url += '&reporterlist_id=' + frm.reporterlist_id.value;
		Dialogs.openPopup(url, 'AddReporters', 1050, 700);
	}

	reporters.removeReporters = function()
	{
		var numChecked = Table.checkIfSelected('reporters');
		if (numChecked == 0) return;

		var rep = numChecked == 1 ? 'reporter' : 'reporters';
		if (!confirm('You are about to remove '+numChecked+' '+rep+' from the reporter list. Continue?'))
		{
			return;
		}
		var frm = document.forms['reporters'];
		frm.cmd.value = 'DeleteItems';
		frm.submit();
	}
	
	reporters.viewReporterList = function()
	{
		var frm = document.forms['reporters'];
		var url = '../index.jsp?ID='+App.getSessionId();
		url += '&cmd=ViewItem&item_id=' + frm.reporterlist_id.value;
		url += '&tab=properties';
		location.replace(url);
	}
	
	reporters.addSelected = function()
	{
		var selected = top['reporters'].Table.getSelected('reporters', null, 'Please select at least one reporter in the list');
		if (selected.length == 0) return;

		var frm = document.forms['addReporters'];
		frm.which.value = 'selected';
		Forms.addHidden(frm, 'items', selected.join(','));
		frm.submit();
	}
	
	reporters.addCurrentPage = function()
	{
		var frm = document.forms['addReporters'];
		frm.which.value = 'page';
		frm.submit();
	}

	reporters.addAll = function()
	{
		var frm = document.forms['addReporters'];
		frm.which.value = 'all';
		frm.submit();
	}

	
	return reporters;
}();

Doc.onLoad(Reporters.initPage);
