

package com.dawidweiss.ant.taskdefs;

import java.io.IOException;
import java.io.Reader;
import java.util.Hashtable;

import org.apache.tools.ant.types.*;
import org.apache.tools.ant.taskdefs.*;
import org.apache.tools.ant.Project;
import java.io.*;
import java.util.*;



/**
 * A liaison class for XSLTProc processor. The processor must be present
 * in path, otherwise an error will occur.
 */
public final class XsltProcLiaison
    implements org.apache.tools.ant.taskdefs.XSLTLiaison {
        
    private File   xsl;
    private Map    parameters = new HashMap();
    private LinkedList catalogs = new LinkedList();
    private Map    xsltprocArguments  = new HashMap();
    private boolean debug = false;

    
    public XsltProcLiaison()
    {
    }
    
    
    public void setStylesheet(File xsl)
        throws Exception
    {
        this.xsl = xsl;
        if (!xsl.canRead())
            throw new IOException("Cannot read XSL stylesheet: " + xsl);
    }
          

    public void addParam(String name, String value)
        throws Exception
    {
    	if ("xsltproc.catalog".equalsIgnoreCase(name)) {
    		if (debug) {
    			System.err.println("XsltProc liason: "
    				+ "adding catalog: " + value);
    		}
            File f = new File( value );
            if (f.isAbsolute()==false)
                throw new RuntimeException("Catalog location must be absolute.");
            
            String path = f.getPath();
            if (File.separatorChar == '\\') {
                // windows?
                path = path.replace('\\', '/');
                path = "file:///" + path;
            } else {
                path = "file://" + path;
            }

            // url-encode the path (this is for XSLTproc)
    		catalogs.add( path );
    	}
    	else
    	if (name.equals("xsltproc.option.debug")) {
    		debug = true;
    	}
    	else
    	if (name.startsWith("xsltproc.option.")) {
    		String option = name.substring("xsltproc.option.".length());
    		if (debug) {
    			System.err.println("XsltProc liason: "
    				+ "adding xsltproc argument: " + option + "=" + value);
    		}
    		xsltprocArguments.put(option, value);
    	}
    	else {
    		if (debug) {
    			System.err.println("XsltProc liason: "
    				+ "adding xsl parameter: " + name + "=" + value);
    		}
        	parameters.put( name, value );
        }
    }


    public void transform(File infile, File outfile)
        throws Exception
    {
        Commandline cmdl = new Commandline();        
        
        cmdl.setExecutable("xsltproc");
        
        if (catalogs.size() > 0) {
        	cmdl.createArgument().setValue("--catalogs");
        }
        
        cmdl.createArgument().setValue("--output");
        cmdl.createArgument().setFile(outfile);
        
        for (Iterator i = xsltprocArguments.keySet().iterator();i.hasNext();)
        {
        	String arg = (String) i.next();
        	String value = (String) xsltprocArguments.get( arg );
        	if (value == null) value = "";
        	
        	cmdl.createArgument().setValue(arg);
			if (!"".equals(value.trim())) {
				cmdl.createArgument().setValue(value);
			}
        }

        for (Iterator i = parameters.keySet().iterator();i.hasNext();)
        {
            String key = (String) i.next();
            String value = (String) parameters.get( key );

            cmdl.createArgument().setValue("--stringparam");
            cmdl.createArgument().setValue(key);
            cmdl.createArgument().setValue(value);
        }
        cmdl.createArgument().setFile(xsl);
        cmdl.createArgument().setFile(infile);

        try
        {
            Execute exe = new Execute();
            exe.setCommandline(cmdl.getCommandline());
            exe.setNewenvironment(true);

            // set environment catalogs, if any
            if (catalogs.size() != 0) {
            	StringBuffer buf = new StringBuffer("XML_CATALOG_FILES=");
            	for (Iterator i = catalogs.iterator();i.hasNext();) {
            		String value = (String) i.next();
            		buf.append( value );
            		if (i.hasNext())
                        buf.append( " " );
            	}

                if (debug) {
    			System.err.println("XsltProc liason: "
    				+ "catalogs: " + buf.toString());
                }
            	exe.setEnvironment(new String [] { buf.toString() });
           	}
            if (debug) {
                System.err.println(exe);
            }
            exe.execute();
        }
        catch (Exception e)
        {
            System.err.println("XSLTProc process has thrown an exception: " + e.toString());
        }
    }

}
